import java.util.* ;
import java.io.* ;
import java.math.* ;
/*
  QS

 Luc Maranget, projet Moscova, Inria Rocquencourt
 
 Copyright 2005 Institut National de Recherche en Informatique et    
 en Automatique.  All rights reserved.  This file is distributed     
 under the terms of the Q Public License version 1.0.          
*/
class Parse {
  private Reader reader ;
  private int next_char ; // -1 = EOF
  private boolean ok ;

  Parse (Reader reader) {
    this.reader = reader ;
    ok = false ;
  }

  private int look() {
    if (ok)
      return next_char ;
    else {
      try {
        ok = true ;
        next_char = reader.read() ;
        if (next_char == '#') {
          next_char = reader.read() ;
          while (next_char != '\n' && next_char != -1)
            next_char = reader.read() ;
        }
        return next_char ;
      } catch (IOException e) {
        System.err.println(e.getMessage()) ;
        System.exit(2) ;
        return -1 ; // Not reached
      }
    }
  }

  private void eat() { ok = false ; }

  private void skipBlanks() {
    while (look() == ' ') {
      eat() ;
    }
  }

  private boolean chiffre(int c) {
    return '0' <= c && c <= '9' ;
  }

  private boolean isEol() {
    int c = look() ;
    return c == '\r' || c == '\n' ;
  }

  private void skipEol() {
    while (isEol()) {
      eat() ;
    }
  }

  private void skipColon() {
    while (look() == ':') {
      eat() ;
    }
  }

  private void skipStar() {
    while (look() == '*') {
      eat() ;
    }
  }

  private StringBuffer buff = new StringBuffer() ;

  private void record(int c) {
    buff.append((char)c) ; eat () ;
  }

  private String trim() {
    String r = buff.toString() ;
    buff.setLength(0) ;
    return r ;
  }

  private String anyInteger() {
    skipBlanks() ;
    int c = look() ;
    if (c != '-' && !chiffre(c)) {
      System.err.println("Integer expected, " + ((char) c) + " found") ;
      System.exit(2) ;
    }
    record(c) ;
    while (chiffre(c = look())) {
      record(c) ;
    }
    return trim() ;
  }

  private BigInteger bigInteger() {
    return new BigInteger(anyInteger()) ;
  }

  private int integer() {
    return Integer.parseInt(anyInteger()) ;
  }

  Eqs eqs() {
    BigInteger n = bigInteger () ;
    int b = integer() ;
    Vector v = new Vector() ;
    skipEol() ;
    while (look() != -1) {
      skipStar() ;
      BigInteger x = bigInteger() ;
      IntList ps = null ;
      skipColon() ;
      while (!isEol()) {
        ps = new IntList (integer(), ps) ;
      }      
      skipEol() ;
      v.addElement(new Eq(x, ps)) ;
    }
    Eq [] eq = new Eq [v.size()] ;
    int i = 0 ;
    for (Enumeration e = v.elements() ; e.hasMoreElements() ; ) {
      eq[i++] = (Eq)e.nextElement() ;
    }
    return new Eqs (n, b, eq) ;
  }
}
