import java.math.* ;
/*
 QS

 Luc Maranget, projet Moscova, Inria Rocquencourt
 
 Copyright 2005 Institut National de Recherche en Informatique et    
 en Automatique.  All rights reserved.  This file is distributed     
 under the terms of the Q Public License version 1.0.          
*/
class T {
  public static void main (String [] argv) {
    Options o = new Options("T", argv) ;
    BigInteger n = o.n ;
    int B = o.B ;
    int verbose = o.verbose ;

    if (B == 0) {
      double b = n.bitLength() ;
      b = b * Math.log(2) ;
      b = b * Math.log(b) ;
      b = Math.exp(0.5*Math.sqrt(b)) ;
      B = 1 + (int)(b) ;
    }

    int [] p = Primes.compute(B) ;

    if (verbose > 1) {
      System.err.println("** Primes **") ;
      for (int i = 0 ; i < p.length ; i++) {
        System.err.print(" " + p[i]) ;
      }
      System.err.println() ;
    }

    /* Try division by factor base */
    int trial = Trial.trial(n, p, 1) ;
    if (trial != 0) {
      if (verbose > -1) {
        System.err.println("Division by factor base") ;
      }
      System.out.println(trial + " * " + n.divide(BigInteger.valueOf(trial))) ;
      return ;
    }

    int [] q = Residue.compute(p, n) ;
    if (verbose > 1) {
      System.err.println("** Residues **") ;
      for (int i = 0 ; i < q.length ; i++) {
        System.err.print(" " + q[i]) ;
      }
      System.err.println() ;      
    }

    /* If factor base is small use naive algorithm */
    if (q.length < 20) {
      int a = Trial.algo(n) ;
      if (a == 0) {
        if (verbose > -1) System.err.println("n is a small prime") ;
        System.out.println("1 * " + n) ;
      } else {
        if (verbose > -1) System.err.println("Found by trial division") ;
        System.out.println(a + " * " + n.divide(BigInteger.valueOf(a))) ;
      }
      return ;
    }

    int r [] = Root.compute(q, n) ;
    if (verbose > 1) {
      System.err.println("** Roots **") ;
      for (int i = 0 ; i < r.length ; i++) {
        System.err.print(" " + r[i]) ;
      }
      System.err.println() ;
    }

    BigInteger sqrtn = Arith.sqrt(n) ;

    /* Look for powers */
    BigInteger root = Trial.roots(n, sqrtn, q[q.length-1]) ;
    if (root != null ) {
      if (verbose > -1) System.err.println("n is a power") ;
      System.out.println(root + " * " + n.divide(root)) ;
      return ;
    }

    /* Try Pollard's rho method */
    if (o.timeout > 0) {
      BigInteger rho = Rho.timeoutRho(o.timeout, n) ;
      if (rho != null) {
        if (verbose > -1) System.err.println("Found by pollard's rho") ;
        System.out.println(rho + " * " + n.divide(rho)) ;
        System.exit(0) ; // To kill timeout thread and exit immediately
      }
    }
    
    // Now we do sieve
    System.out.println(n + " " + B) ;
    int step = o.T ;
    int b = q.length ;
    if (step == 0) {
      if (b < 1500) {
        step = 64 * 1024 ;
      } else if (b < 2500) {
        step = 512 * 1024 ;
      } else if (B < 5000) {
        step = 1024 * 1024 ;
      } else {
        step = 2 * 1024 * 1024 ;
      }
    }

    if (verbose > -1) {
      System.err.println("n=" + n) ;
      System.err.println("B=" + B) ;
      System.err.println("b=" + q.length) ;
      System.err.println("step=" + step/1024) ;
    }


    BigInteger v = sqrtn.add(BigInteger.ONE) ;
    BigInteger w = sqrtn.subtract(BigInteger.valueOf(step)) ;
    Collector collector = new Collector(10, q) ;


    Sieve s0 =
      o.simple ?
      new Simple(q, r, n, step, B, collector, o) :
      new Sieve(q, r, n, step, B, collector, o) ;
      
    BigInteger bstep = BigInteger.valueOf(step) ;
    int nsteps = 0 ;
    for( ; ; ) {
      nsteps++ ;
      if (s0.zyva(v, (nsteps-1) / 2)) break ;
      v = v.add(bstep ) ;
      nsteps++ ;
      if (s0.zyva(w, -(nsteps / 2))) break ;
      w = w.subtract(bstep) ;
    }
    /*
      for ( ; ; ) {
      v = v.add(BigInteger.valueOf(step)) ;
      cell.set(v) ;
      if (collector.isOver()) break ;
      w = w.subtract(BigInteger.valueOf(step)) ;
      cell.set(w) ;
      if (collector.isOver()) break ;
      }
    */
    s0.finalEfficiency(nsteps) ;
    collector.efficiency() ;
    System.err.println("Scanned: " + ((double)nsteps * step)/(1024.0 * 1024.0) + " Mo") ;
  }
}
